#include <STDIO.H>
#include <DOS.H>

void setPixel(int x, int y, unsigned char c) {
  asm {
    mov ah, 0x0C;
    mov al, c;
    xor bh, bh;
    mov cx, x;
    mov dx, y;
    int 0x10;
  }
}

void setScreenMode(void) {
  int i;
  static const unsigned char pal[] =
  {  0,   0,   0,
     0, 168, 168,
   252,  84, 252,
   252, 252, 252,
     0,   0, 168,
    84, 252,  84,
   252,  84,  84,
   252, 252,  84,
    84,  84,  84,
    84,  84, 252,
   168,   0, 168,
    84, 252, 252,
   168,  84,   0,
     0, 168,   0,
   168,   0,   0,
   168, 168, 168
  };

  asm {
    mov ax, 0x0012;
    int 0x10;
  }

  //set palette
  for (i = 0; i < 8; i++) {
    if (i == 6) {
      outportb(0x3C8, 20);
    }
    else {
      outportb(0x3C8, i);
    }
    outportb(0x3C9, pal[i*3+0] >> 2);
    outportb(0x3C9, pal[i*3+1] >> 2);
    outportb(0x3C9, pal[i*3+2] >> 2);
  }
  for (i = 8; i < 16; i++) {
    outportb(0x3C8, i + 48);
    outportb(0x3C9, pal[i*3+0] >> 2);
    outportb(0x3C9, pal[i*3+1] >> 2);
    outportb(0x3C9, pal[i*3+2] >> 2);
  }
}

int main(int argc, char* argv[]) {
  int x, y;
  int tx, ty;
  FILE* tilesFile;
  unsigned char mask[2*16];
  unsigned char tile[16*16];

  if (argc < 2) {
    printf("Mega Man III tile viewer by Frenkel, 23-Aug-2008.\n");
    printf("Usage: tiles DECOMPRESSEDTILESFILE\n");
    return -1;
  }

  tilesFile = fopen(argv[1], "rb");
  if (tilesFile == NULL) {
    printf("Mega Man III tile viewer by Frenkel, 23-Aug-2008.\n");
    printf("Can't find %s!\n", argv[1]);
    return -1;
  }

  setScreenMode();

  //Draw tiles
  for (ty = 0; ty < 16; ty++) {
    for (tx = 0; tx < 16; tx++) {
      fread(tile, 16*16, 1, tilesFile);
      for (y = 0; y < 16; y++) {
	for (x = 0; x < 16; x++) {
	  setPixel(x+tx*16, y+ty*16, tile[y*16+x]);
	}
      }
    }
  }

  //Draw mask
  for (ty = 0; ty < 16; ty++) {
    for (tx = 0; tx < 16; tx++) {
      fread(mask, 2*16, 1, tilesFile);
      for (y = 0; y < 16; y++) {
	for (x = 0; x < 2; x++) {
	  pokeb(0xA000,(y+ty*16)*80+x+tx*2+16*2, mask[y*2+x]);
	}
      }
    }
  }

  fclose(tilesFile);

  asm {
    xor ah, ah;
    int 0x16;

    mov ax, 0x0003;
    int 0x10;
  }
  printf("Mega Man III tile viewer by Frenkel, 23-Aug-2008.\n");
  return 0;
}